#!/bin/zsh

# Copyright 2016 Paul Hänsch
#
# This file is part of shcgi.
# 
# shcgi is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# shcgi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with shcgi.  If not, see <http://www.gnu.org/licenses/>. 


unset length date file suffix
file="$1"
date="$(stat -c %Y "$file")"

# allow overriding magic file recognition
# don't use this if `file` returns type correctly
declare -A suffix
suffix[css]="text/css"

[ -n "$HTTP_IF_MODIFIED_SINCE" ] && cachedate="$(date -d "$HTTP_IF_MODIFIED_SINCE" +%s)"

if [ -x "$file" -o \! -r "$file" -o \! -f "$file" ]; then
  printf 'Status:403 Forbidden\r\n\r\n'
elif [ "$date" = "$cachedate" ]; then
  debug "Delegating to client cache"
  printf 'Status:304 Not Modified\r\n'
  printf 'Last-Modified: %s\r\n\r\n' "$(date -Rd "@$date")"
else
  length="$(stat -c %s "$file")"
  magic="${suffix[${file##*.}]:-$(file -bi "$file")}"

  printf 'Last-Modified: %s\r\n' "$(date -Rd "@$date")"
  printf 'Content-Type: %s\r\n' "${magic:-all/all}"
  printf 'Content-Length: %s\r\n' "$length"
  printf '\r\n'

  cat "$file"
fi

